/*
 * ** header v3.0
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2018 Research Group Biomedical Physics,
 * Max-Planck-Institute for Dynamics and Self-Organization Göttingen
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 * ** end header
 */
package org.caosdb.server.transaction;

import org.caosdb.server.database.BackendTransaction;
import org.caosdb.server.database.access.Access;
import org.caosdb.server.database.misc.RollBackHandler;
import org.caosdb.server.database.misc.TransactionBenchmark;

public interface TransactionInterface {

  public void execute() throws Exception;

  public default TransactionBenchmark getTransactionBenchmark() {
    return TransactionBenchmark.getRootInstance().getBenchmark(getClass());
  }

  public default <K extends BackendTransaction> K execute(K t, Access access) {
    final RollBackHandler handler = (RollBackHandler) access.getHelper("RollBack");
    handler.append(t);
    t.setAccess(access);

    t.setTransactionBenchmark(getTransactionBenchmark().getBenchmark(t.getClass()));
    t.executeTransaction();
    return t;
  }
}
